@@ -7,13 +7,14 @@
 /*	#include "bounce_service.h"
 /*
 /*	int     bounce_notify_verp(flags, service, queue_name, queue_id, sender,
-/*					verp_delims)
+/*					dsn_envid, dsn_ret, verp_delims)
 /*	int	flags;
 /*	char	*queue_name;
 /*	char	*queue_id;
 /*	char	*sender;
+/*	char	*dsn_envid;
+/*	int	dsn_ret;
 /*	char	*verp_delims;
-/*	int	flush;
 /* DESCRIPTION
 /*	This module implements the server side of the bounce_notify()
 /*	(send bounce message) request. The logfile
@@ -29,9 +30,7 @@
 /*	When a bounce is sent, the sender address is the empty
 /*	address.
 /* DIAGNOSTICS
-/*	Fatal error: error opening existing file. Warnings: corrupt
-/*	message file. A corrupt message is saved to the "corrupt"
-/*	queue for further inspection.
+/*	Fatal error: error opening existing file.
 /* SEE ALSO
 /*	bounce(3) basic bounce service client interface
 /* LICENSE
@@ -72,6 +71,7 @@
 #include <mail_error.h>
 #include <verp_sender.h>
 #include <bounce.h>
+#include <dsn_mask.h>
 
 /* Application-specific. */
 
@@ -79,11 +79,12 @@
 
 #define STR vstring_str
 
-/* bounce_notify_verp - send a bounce */
+/* bounce_notify_verp - send a bounce, VERP style */
 
 int     bounce_notify_verp(int flags, char *service, char *queue_name,
 			           char *queue_id, char *encoding,
-			           char *recipient, char *verp_delims)
+			           char *recipient, char *dsn_envid,
+			           int dsn_ret, char *verp_delims)
 {
     char   *myname = "bounce_notify_verp";
     BOUNCE_INFO *bounce_info;
@@ -108,12 +109,10 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
      * Initialize. Open queue file, bounce log, etc.
      */
     bounce_info = bounce_mail_init(service, queue_name, queue_id,
-				   encoding, BOUNCE_MSG_FAIL);
+				   encoding, dsn_envid, BOUNCE_REPORT_FAIL);
 
 #define NULL_SENDER		MAIL_ADDR_EMPTY	/* special address */
 #define NULL_TRACE_FLAGS	0
-#define BOUNCE_HEADERS		1
-#define BOUNCE_ALL		0
 
     /*
      * A non-bounce message was returned. Send a single bounce, one per
@@ -122,51 +121,58 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
     while (bounce_log_read(bounce_info->log_handle) != 0) {
 
 	/*
-	 * Notify the originator.
+	 * Notify the originator, subject to DSN NOTIFY restrictions.
 	 */
-	verp_sender(verp_buf, verp_delims, recipient,
-		    bounce_info->log_handle->recipient);
-	if ((bounce = post_mail_fopen_nowait(NULL_SENDER, STR(verp_buf),
-					     CLEANUP_FLAG_MASK_INTERNAL,
-					     NULL_TRACE_FLAGS)) != 0) {
+	if (bounce_info->log_handle->rcpt.dsn_notify != 0	/* compat */
+	&& (bounce_info->log_handle->rcpt.dsn_notify & DSN_NOTIFY_FAILURE) == 0) {
+	    bounce_status = 0;
+	} else {
+	    verp_sender(verp_buf, verp_delims, recipient,
+			bounce_info->log_handle->rcpt.address);
+	    if ((bounce = post_mail_fopen_nowait(NULL_SENDER, STR(verp_buf),
+						 CLEANUP_FLAG_MASK_INTERNAL,
+						 NULL_TRACE_FLAGS)) != 0) {
+
+		/*
+		 * Send the bounce message header, some boilerplate text that
+		 * pretends that we are a polite mail system, the text with
+		 * reason for the bounce, and a copy of the original message.
+		 */
+		if (bounce_header(bounce, bounce_info, STR(verp_buf)) == 0
+		    && bounce_boilerplate(bounce, bounce_info) == 0
+		    && bounce_recipient_log(bounce, bounce_info) == 0
+		    && bounce_header_dsn(bounce, bounce_info) == 0
+		    && bounce_recipient_dsn(bounce, bounce_info) == 0)
+		    bounce_original(bounce, bounce_info, dsn_ret ?
+				    dsn_ret : DSN_RET_FULL);
+		bounce_status = post_mail_fclose(bounce);
+	    } else
+		bounce_status = 1;
 
 	    /*
-	     * Send the bounce message header, some boilerplate text that
-	     * pretends that we are a polite mail system, the text with
-	     * reason for the bounce, and a copy of the original message.
+	     * Stop at the first sign of trouble, instead of making the
+	     * problem worse.
 	     */
-	    if (bounce_header(bounce, bounce_info, STR(verp_buf)) == 0
-		&& bounce_boilerplate(bounce, bounce_info) == 0
-		&& bounce_recipient_log(bounce, bounce_info) == 0
-		&& bounce_header_dsn(bounce, bounce_info) == 0
-		&& bounce_recipient_dsn(bounce, bounce_info) == 0)
-		bounce_original(bounce, bounce_info, BOUNCE_ALL);
-	    bounce_status = post_mail_fclose(bounce);
-	} else
-	    bounce_status = 1;
+	    if (bounce_status != 0)
+		break;
+
+	    /*
+	     * Optionally, mark this recipient as done.
+	     */
+	    if (flags & BOUNCE_FLAG_DELRCPT)
+		bounce_delrcpt_one(bounce_info);
+	}
 
 	/*
-	 * Stop at the first sign of trouble, instead of making the problem
-	 * worse.
-	 */
-	if (bounce_status != 0)
-	    break;
-
-	/*
-	 * Optionally, mark this recipient as done.
-	 */
-	if (flags & BOUNCE_FLAG_DELRCPT)
-	    bounce_delrcpt_one(bounce_info);
-
-	/*
-	 * Optionally, send a postmaster notice.
+	 * Optionally, send a postmaster notice, subject to notify_classes
+	 * restrictions.
 	 * 
 	 * This postmaster notice is not critical, so if it fails don't
 	 * retransmit the bounce that we just generated, just log a warning.
 	 */
-#define WANT_IF_BOUNCE ((notify_mask & MAIL_ERROR_BOUNCE))
+#define SEND_POSTMASTER_SINGLE_BOUNCE_NOTICE (notify_mask & MAIL_ERROR_BOUNCE)
 
-	if (WANT_IF_BOUNCE) {
+	if (SEND_POSTMASTER_SINGLE_BOUNCE_NOTICE) {
 
 	    /*
 	     * Send the text with reason for the bounce, and the headers of
@@ -184,7 +190,7 @@ int     bounce_notify_verp(int flags, char *service, char *queue_name,
 		    && bounce_recipient_log(bounce, bounce_info) == 0
 		    && bounce_header_dsn(bounce, bounce_info) == 0
 		    && bounce_recipient_dsn(bounce, bounce_info) == 0)
-		    bounce_original(bounce, bounce_info, BOUNCE_HEADERS);
+		    bounce_original(bounce, bounce_info, DSN_RET_HDRS);
 		postmaster_status = post_mail_fclose(bounce);
 	    } else
 		postmaster_status = 1;
