@@ -90,7 +90,7 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
     char   *myname = "deliver_mailbox_file";
     char   *spool_dir;
     char   *mailbox;
-    DSN_VSTRING *why;
+    DSN_BUF *why = state.msg_attr.why;
     MBOX   *mp;
     int     mail_copy_status;
     int     deliver_status;
@@ -113,10 +113,10 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
     /*
      * Don't deliver trace-only requests.
      */
-    if (DEL_REQ_TRACE_ONLY(state.request->flags))
-	return (sent(BOUNCE_FLAGS(state.request),
-		     SENT_ATTR(state.msg_attr, "2.0.0"),
-		     "delivers to mailbox"));
+    if (DEL_REQ_TRACE_ONLY(state.request->flags)) {
+	dsb_simple(why, "2.0.0", "delivers to mailbox");
+	return (sent(BOUNCE_FLAGS(state.request), SENT_ATTR(state.msg_attr)));
+    }
 
     /*
      * Initialize. Assume the operation will fail. Set the delivered
@@ -124,9 +124,8 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
      */
     if (vstream_fseek(state.msg_attr.fp, state.msg_attr.offset, SEEK_SET) < 0)
 	msg_fatal("seek message file %s: %m", VSTREAM_PATH(state.msg_attr.fp));
-    state.msg_attr.delivered = state.msg_attr.recipient;
+    state.msg_attr.delivered = state.msg_attr.rcpt.address;
     mail_copy_status = MAIL_COPY_STAT_WRITE;
-    why = dsn_vstring_alloc(100);
     if (*var_home_mailbox) {
 	spool_dir = 0;
 	mailbox = concatenate(usr_attr.home, "/", var_home_mailbox, (char *) 0);
@@ -192,9 +191,8 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
 	    set_eugid(usr_attr.uid, usr_attr.gid);
 	if (S_ISREG(st.st_mode) == 0) {
 	    vstream_fclose(mp->fp);
-	    dsn_vstring_update(why, "5.2.0",
-			       "destination %s is not a regular file",
-			       mailbox);
+	    dsb_simple(why, "5.2.0",
+		       "destination %s is not a regular file", mailbox);
 	} else {
 	    end = vstream_fseek(mp->fp, (off_t) 0, SEEK_END);
 	    mail_copy_status = mail_copy(COPY_ATTR(state.msg_attr), mp->fp,
@@ -212,20 +210,21 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
     if (mail_copy_status & MAIL_COPY_STAT_CORRUPT) {
 	deliver_status = DEL_STAT_DEFER;
     } else if (mail_copy_status != 0) {
-	deliver_status = (DSN_CLASS(why->dsn) == '4' ?
-			  defer_append : bounce_append)
-	    (BOUNCE_FLAGS(state.request),
-	     BOUNCE_ATTR(state.msg_attr, DSN_CODE(why->dsn)),
-	     "cannot update mailbox %s for user %s. %s",
-	     mailbox, state.msg_attr.user, vstring_str(why->vstring));
+	vstring_sprintf_prepend(why->reason,
+				"cannot update mailbox %s for user %s. ",
+				mailbox, state.msg_attr.user);
+	deliver_status =
+	    (STR(why->status)[0] == '4' ?
+	     defer_append : bounce_append)
+	    (BOUNCE_FLAGS(state.request), BOUNCE_ATTR(state.msg_attr));
     } else {
+	dsb_simple(why, "2.0.0", "delivered to mailbox");
 	deliver_status = sent(BOUNCE_FLAGS(state.request),
-			      SENT_ATTR(state.msg_attr, "2.0.0"),
-			      "delivered to mailbox");
+			      SENT_ATTR(state.msg_attr));
 	if (var_biff) {
 	    biff = vstring_alloc(100);
 	    vstring_sprintf(biff, "%s@%ld", usr_attr.logname, (long) end);
-	    biff_notify(vstring_str(biff), VSTRING_LEN(biff) + 1);
+	    biff_notify(STR(biff), VSTRING_LEN(biff) + 1);
 	    vstring_free(biff);
 	}
     }
@@ -234,7 +233,6 @@ static int deliver_mailbox_file(LOCAL_STATE state, USER_ATTR usr_attr)
      * Clean up.
      */
     myfree(mailbox);
-    dsn_vstring_free(why);
     return (deliver_status);
 }
 
@@ -268,9 +266,9 @@ int     deliver_mailbox(LOCAL_STATE state, USER_ATTR usr_attr, int *statusp)
      * Delegate mailbox delivery to another message transport.
      */
     if (*var_mailbox_transport) {
+	state.msg_attr.rcpt.offset = -1L;
 	*statusp = deliver_pass(MAIL_CLASS_PRIVATE, var_mailbox_transport,
-				state.request, state.msg_attr.orig_rcpt,
-				state.msg_attr.recipient, -1L);
+				state.request, &state.msg_attr.rcpt);
 	return (YES);
     }
 
