@@ -46,7 +46,7 @@
 /*	Delivery is deferred in case of failure.
 /* .sp
 /*	This feature is available as of Postfix 2.2.
-/* .IP "\fBeol=string\fR (optional, default: \fB\en\fR)"
+/* .IP "\fBeol=\fIstring\fR (optional, default: \fB\en\fR)"
 /*	The output record delimiter. Typically one would use either
 /*	\fB\er\en\fR or \fB\en\fR. The usual C-style backslash escape
 /*	sequences are recognized: \fB\ea \eb \ef \en \er \et \ev
@@ -106,6 +106,33 @@
 /*	Prepend "\fB>\fR" to lines starting with "\fBFrom \fR". This is expected
 /*	by, for example, \fBUUCP\fR software.
 /* .RE
+/* .IP "\fBnull_sender\fR=\fIreplacement\fR (default: MAILER-DAEMON)"
+/*	Replace the null sender address, which is typically used
+/*	for delivery status notifications, with the specified text
+/*	when expanding the \fB$sender\fR command-line macro, and
+/*	when generating a From_ or Return-Path: message header.
+/*
+/*	If the null sender replacement text is a non-empty string
+/*	then it is affected by the \fBq\fR flag for address quoting
+/*	in command-line arguments.
+/*
+/*	The null sender replacement text may be empty; this form
+/*	is recommended for content filters that feed mail back into
+/*	Postfix. The empty sender address is not affected by the
+/*	\fBq\fR flag for address quoting in command-line arguments.
+/* .sp
+/*	Caution: a null sender address is easily mis-parsed by
+/*	naive software. For example, when the \fBpipe\fR(8) daemon
+/*	executes a command such as:
+/*
+/* .ti +4
+/*	command -f$sender -- $recipient
+/*
+/*	the command will mis-parse the -f option value when the
+/*	sender address is a null string.  For correct parsing,
+/*	specify \fB$sender\fR as an argument by itself.
+/* .sp
+/*	This feature is available with Postfix 2.3 and later.
 /* .IP "\fBsize\fR=\fIsize_limit\fR (optional)"
 /*	Messages greater in size than this limit (in bytes) will be bounced
 /*	back to the sender.
@@ -162,7 +189,7 @@
 /*	\fIuser+foo\fR.
 /* .sp
 /*	A command-line argument that contains \fB${\fBmailbox\fR}\fR
-/*	expands into as many command-line arguments as there are recipients.
+/*	expands to as many command-line arguments as there are recipients.
 /* .sp
 /*	This information is modified by the \fBu\fR flag for case folding.
 /* .IP \fB${\fBnexthop\fR}\fR
@@ -173,7 +200,7 @@
 /*	This macro expands to the complete recipient address.
 /* .sp
 /*	A command-line argument that contains \fB${\fBrecipient\fR}\fR
-/*	expands into as many command-line arguments as there are recipients.
+/*	expands to as many command-line arguments as there are recipients.
 /* .sp
 /*	This information is modified by the \fBhqu\fR flags for quoting
 /*	and case folding.
@@ -184,7 +211,7 @@
 /* .sp
 /*	This is available in Postfix 2.2 and later.
 /* .IP \fB${\fBsasl_sender\fR}\fR
-/*	This macro expands to the SASL sender name (i.e. the original 
+/*	This macro expands to the SASL sender name (i.e. the original
 /*	submitter as per RFC 2554) used during the reception of the message.
 /* .sp
 /*	This is available in Postfix 2.2 and later.
@@ -195,7 +222,10 @@
 /* .sp
 /*	This is available in Postfix 2.2 and later.
 /* .IP \fB${\fBsender\fR}\fR
-/*	This macro expands to the envelope sender address.
+/*	This macro expands to the envelope sender address. By default,
+/*	the null sender address expands to MAILER-DAEMON; this can
+/*	be changed with the \fBnull_sender\fR attribute, as described
+/*	above.
 /* .sp
 /*	This information is modified by the \fBq\fR flag for quoting.
 /* .IP \fB${\fBsize\fR}\fR
@@ -363,6 +393,8 @@
 #include <quote_822_local.h>
 #include <flush_clnt.h>
 #include <dsn_util.h>
+#include <dsn_buf.h>
+#include <sys_exits.h>
 
 /* Single server skeleton. */
 
@@ -440,6 +472,7 @@ typedef struct {
     int     flags;			/* mail_copy() flags */
     char   *exec_dir;			/* working directory */
     VSTRING *eol;			/* output record delimiter */
+    VSTRING *null_sender;		/* null sender expansion */
     off_t   size_limit;			/* max size in bytes we will accept */
 } PIPE_ATTR;
 
@@ -692,6 +725,7 @@ static void get_service_attr(PIPE_ATTR *attr, char **argv)
     attr->flags = 0;
     attr->exec_dir = 0;
     attr->eol = vstring_strcpy(vstring_alloc(1), "\n");
+    attr->null_sender = vstring_strcpy(vstring_alloc(1), MAIL_ADDR_MAIL_DAEMON);
     attr->size_limit = 0;
 
     /*
@@ -776,6 +810,13 @@ static void get_service_attr(PIPE_ATTR *attr, char **argv)
 	    unescape(attr->eol, *argv + sizeof("eol=") - 1);
 	}
 
+	/*
+	 * null_sender=string
+	 */
+	else if (strncasecmp("null_sender=", *argv, sizeof("eol=") - 1) == 0) {
+	    vstring_strcpy(attr->null_sender, *argv + sizeof("null_sender=") - 1);
+	}
+
 	/*
 	 * size=max_message_size (in bytes)
 	 */
@@ -832,12 +873,13 @@ static void get_service_attr(PIPE_ATTR *attr, char **argv)
 
 static int eval_command_status(int command_status, char *service,
 			             DELIVER_REQUEST *request, VSTREAM *src,
-			               const char *dsn, const char *text)
+			               DSN_BUF *why)
 {
     RECIPIENT *rcpt;
     int     status;
     int     result = 0;
     int     n;
+    DSN     dsn;
 
     /*
      * Depending on the result, bounce or defer the message, and mark the
@@ -845,13 +887,14 @@ static int eval_command_status(int command_status, char *service,
      */
     switch (command_status) {
     case PIPE_STAT_OK:
+	dsb_update(why, "2.0.0", "relayed", DSB_SKIP_RMTA, DSB_SKIP_REPLY,
+		   "delivered via %s service", service);
+	(void) DSN_FROM_DSN_BUF(&dsn, why);
 	for (n = 0; n < request->rcpt_list.len; n++) {
 	    rcpt = request->rcpt_list.info + n;
 	    status = sent(DEL_REQ_TRACE_FLAGS(request->flags),
-			  request->queue_id, rcpt->orig_addr,
-			  rcpt->address, rcpt->offset, service,
-			  "2.0.0", request->arrival_time,
-			  "%s", request->nexthop);
+			  request->queue_id, request->arrival_time, rcpt,
+			  service, &dsn);
 	    if (status == 0 && (request->flags & DEL_REQ_FLAG_SUCCESS))
 		deliver_completed(src, rcpt->offset);
 	    result |= status;
@@ -859,14 +902,14 @@ static int eval_command_status(int command_status, char *service,
 	break;
     case PIPE_STAT_BOUNCE:
     case PIPE_STAT_DEFER:
-	if (dsn[0] != '4') {
+	(void) DSN_FROM_DSN_BUF(&dsn, why);
+	if (STR(why->status)[0] != '4') {
 	    for (n = 0; n < request->rcpt_list.len; n++) {
 		rcpt = request->rcpt_list.info + n;
 		status = bounce_append(DEL_REQ_TRACE_FLAGS(request->flags),
-				       request->queue_id, rcpt->orig_addr,
-				       rcpt->address, rcpt->offset, service,
-				       dsn, request->arrival_time,
-				       "%s", text);
+				       request->queue_id,
+				       request->arrival_time, rcpt,
+				       service, &dsn);
 		if (status == 0)
 		    deliver_completed(src, rcpt->offset);
 		result |= status;
@@ -875,20 +918,21 @@ static int eval_command_status(int command_status, char *service,
 	    for (n = 0; n < request->rcpt_list.len; n++) {
 		rcpt = request->rcpt_list.info + n;
 		result |= defer_append(DEL_REQ_TRACE_FLAGS(request->flags),
-				       request->queue_id, rcpt->orig_addr,
-				       rcpt->address, rcpt->offset, service,
-				       dsn, request->arrival_time,
-				       "%s", text);
+				       request->queue_id,
+				       request->arrival_time, rcpt,
+				       service, &dsn);
 	    }
 	}
 	break;
     case PIPE_STAT_CORRUPT:
+	/* XXX DSN should we send something? */
 	result |= DEL_STAT_DEFER;
 	break;
     default:
 	msg_panic("eval_command_status: bad status %d", command_status);
 	/* NOTREACHED */
     }
+
     return (result);
 }
 
@@ -900,37 +944,23 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
     static PIPE_PARAMS conf;
     static PIPE_ATTR attr;
     RECIPIENT_LIST *rcpt_list = &request->rcpt_list;
-    DSN_VSTRING *why = dsn_vstring_alloc(100);
+    DSN_BUF *why = dsb_create();
+    DSN     dsn;
     VSTRING *buf;
     ARGV   *expanded_argv = 0;
     int     deliver_status;
     int     command_status;
     ARGV   *export_env;
+    const char *sender;
 
 #define DELIVER_MSG_CLEANUP() { \
-	dsn_vstring_free(why); \
+	dsb_free(why); \
 	if (expanded_argv) argv_free(expanded_argv); \
     }
 
     if (msg_verbose)
 	msg_info("%s: from <%s>", myname, request->sender);
 
-    /*
-     * First of all, replace an empty sender address by the mailer daemon
-     * address. The resolver already fixes empty recipient addresses.
-     * 
-     * XXX Should sender and recipient be transformed into external (i.e.
-     * quoted) form? Problem is that the quoting rules are transport
-     * specific. Such information must evidently not be hard coded into
-     * Postfix, but would have to be provided in the form of lookup tables.
-     */
-    if (request->sender[0] == 0) {
-	buf = vstring_alloc(100);
-	canon_addr_internal(buf, MAIL_ADDR_MAIL_DAEMON);
-	myfree(request->sender);
-	request->sender = vstring_export(buf);
-    }
-
     /*
      * Sanity checks. The get_service_params() and get_service_attr()
      * routines also do some sanity checks. Look up service attributes and
@@ -950,9 +980,9 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
      * The D flag cannot be specified for multi-recipient deliveries.
      */
     if ((attr.flags & MAIL_COPY_DELIVERED) && (rcpt_list->len > 1)) {
+	dsb_simple(why, "4.3.5", "mail system configuration error");
 	deliver_status = eval_command_status(PIPE_STAT_DEFER, service,
-					     request, request->fp, "4.3.5",
-					     "mailer configuration error");
+					     request, request->fp, why);
 	msg_warn("pipe flag `D' requires %s_destination_recipient_limit = 1",
 		 service);
 	DELIVER_MSG_CLEANUP();
@@ -963,9 +993,9 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
      * The O flag cannot be specified for multi-recipient deliveries.
      */
     if ((attr.flags & MAIL_COPY_ORIG_RCPT) && (rcpt_list->len > 1)) {
+	dsb_simple(why, "4.3.5", "mail system configuration error");
 	deliver_status = eval_command_status(PIPE_STAT_DEFER, service,
-					     request, request->fp, "4.3.5",
-					     "mailer configuration error");
+					     request, request->fp, why);
 	msg_warn("pipe flag `O' requires %s_destination_recipient_limit = 1",
 		 service);
 	DELIVER_MSG_CLEANUP();
@@ -979,10 +1009,9 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
 	if (msg_verbose)
 	    msg_info("%s: too big: size_limit = %ld, request->data_size = %ld",
 		     myname, (long) attr.size_limit, request->data_size);
-
+	dsb_simple(why, "5.2.3", "message too large");
 	deliver_status = eval_command_status(PIPE_STAT_BOUNCE, service,
-					     request, request->fp, "5.2.3",
-					     "message too large");
+					     request, request->fp, why);
 	DELIVER_MSG_CLEANUP();
 	return (deliver_status);
     }
@@ -996,13 +1025,12 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
 	int     n;
 
 	deliver_status = 0;
+	dsb_simple(why, "2.0.0", "delivers to command: %s", attr.command[0]);
 	for (n = 0; n < request->rcpt_list.len; n++) {
 	    rcpt = request->rcpt_list.info + n;
 	    status = sent(DEL_REQ_TRACE_FLAGS(request->flags),
-			  request->queue_id, rcpt->orig_addr,
-			  rcpt->address, rcpt->offset, service,
-			  "2.0.0", request->arrival_time,
-			  "delivers to command: %s", attr.command[0]);
+			  request->queue_id, request->arrival_time,
+			  rcpt, service, DSN_FROM_DSN_BUF(&dsn, why));
 	    if (status == 0 && (request->flags & DEL_REQ_FLAG_SUCCESS))
 		deliver_completed(request->fp, rcpt->offset);
 	    deliver_status |= status;
@@ -1020,12 +1048,16 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
     if (vstream_fseek(request->fp, request->data_offset, SEEK_SET) < 0)
 	msg_fatal("seek queue file %s: %m", VSTREAM_PATH(request->fp));
 
+    /*
+     * A non-empty null sender replacement is subject to the 'q' flag.
+     */
     buf = vstring_alloc(10);
-    if (attr.flags & PIPE_OPT_QUOTE_LOCAL) {
-	quote_822_local(buf, request->sender);
+    sender = *request->sender ? request->sender : STR(attr.null_sender);
+    if (*sender && (attr.flags & PIPE_OPT_QUOTE_LOCAL)) {
+	quote_822_local(buf, sender);
 	dict_update(PIPE_DICT_TABLE, PIPE_DICT_SENDER, STR(buf));
     } else
-	dict_update(PIPE_DICT_TABLE, PIPE_DICT_SENDER, request->sender);
+	dict_update(PIPE_DICT_TABLE, PIPE_DICT_SENDER, sender);
     if (attr.flags & PIPE_OPT_FOLD_HOST) {
 	vstring_strcpy(buf, request->nexthop);
 	lowercase(STR(buf));
@@ -1052,9 +1084,9 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
 
     if ((expanded_argv = expand_argv(service, attr.command,
 				     rcpt_list, attr.flags)) == 0) {
+	dsb_simple(why, "4.3.5", "mail system configuration error");
 	deliver_status = eval_command_status(PIPE_STAT_DEFER, service,
-					     request, request->fp, "4.3.5",
-					     "mailer configuration error");
+					     request, request->fp, why);
 	DELIVER_MSG_CLEANUP();
 	return (deliver_status);
     }
@@ -1063,7 +1095,7 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
     command_status = pipe_command(request->fp, why,
 				  PIPE_CMD_UID, attr.uid,
 				  PIPE_CMD_GID, attr.gid,
-				  PIPE_CMD_SENDER, request->sender,
+				  PIPE_CMD_SENDER, sender,
 				  PIPE_CMD_COPY_FLAGS, attr.flags,
 				  PIPE_CMD_ARGV, expanded_argv->argv,
 				  PIPE_CMD_TIME_LIMIT, conf.time_limit,
@@ -1076,8 +1108,7 @@ static int deliver_message(DELIVER_REQUEST *request, char *service, char **argv)
     argv_free(export_env);
 
     deliver_status = eval_command_status(command_status, service, request,
-					 request->fp, DSN_CODE(why->dsn),
-					 vstring_str(why->vstring));
+					 request->fp, why);
 
     /*
      * Clean up.
