@@ -5,60 +5,18 @@
 /*	Postfix configuration utility
 /* SYNOPSIS
 /* .fi
-/*	\fBpostconf\fR [\fB-dhnv\fR] [\fB-c \fIconfig_dir\fR]
+/*	\fBpostconf\fR [\fB-dhmlnv\fR] [\fB-c \fIconfig_dir\fR]
 /*	[\fIparameter ...\fR]
 /*
-/*	\fBpostconf\fR [\fB-aAmlv\fR] [\fB-c \fIconfig_dir\fR]
-/*
 /*	\fBpostconf\fR [\fB-ev\fR] [\fB-c \fIconfig_dir\fR]
 /*	[\fIparameter=value ...\fR]
-/*
-/*	\fBpostconf\fR [\fB-btv\fR] [\fB-c \fIconfig_dir\fR] [\fItemplate_file\fR]
 /* DESCRIPTION
-/*	The \fBpostconf\fR(1) command displays the actual values
-/*	of configuration parameters, changes configuration parameter
-/*	values, or displays other configuration information about
-/*	the Postfix mail system.
+/*	The \fBpostconf\fR(1) command prints the actual value of
+/*	\fIparameter\fR (all known parameters by default) one
+/*	parameter per line, changes its value, or prints other
+/*	information about the Postfix mail system.
 /*
 /*	Options:
-/* .IP \fB-a\fR
-/*	List the available SASL server plug-in types.  The SASL
-/*	plug-in type is selected with the \fBsmtpd_sasl_type\fR
-/*	configuration parameter by specifying one of the names
-/*	listed below.
-/* .RS
-/* .IP \fBcyrus\fR
-/*	This server plug-in is available when Postfix is built with
-/*	Cyrus SASL support.
-/* .IP \fBdovecot\fR
-/*	This server plug-in requires the Dovecot authentication
-/*	server.
-/* .RE
-/* .IP
-/*	This feature is available with Postfix 2.3 and later.
-/* .IP \fB-A\fR
-/*	List the available SASL client plug-in types.  The SASL
-/*	plug-in type is selected with the \fBsmtp_sasl_type\fR or
-/*	\fBlmtp_sasl_type\fR configuration parameters by specifying
-/*	one of the names listed below.
-/* .RS
-/* .IP \fBcyrus\fR
-/*	This client plug-in is available when Postfix is built with
-/*	Cyrus SASL support.
-/* .RE
-/* .IP
-/*	This feature is available with Postfix 2.3 and later.
-/* .IP "\fB-b\fR [\fItemplate_file\fR]"
-/*	Display the message text that appears at the beginning of
-/*	delivery status notification (DSN) messages, with $\fBname\fR
-/*	expressions replaced by actual values.  To override the
-/*	built-in message text, specify a template file at the end
-/*	of the command line, or specify a template file in main.cf
-/*	with the \fBbounce_template_file\fR parameter.
-/*	To force selection of the built-in message text templates,
-/*	specify an empty template file name (in shell language: "").
-/*
-/*	This feature is available with Postfix 2.3 and later.
 /* .IP "\fB-c \fIconfig_dir\fR"
 /*	The \fBmain.cf\fR configuration file is in the named directory
 /*	instead of the default configuration directory.
@@ -146,7 +104,7 @@
 /* .IP "\fBtcp\fR (read-only)"
 /*	Perform lookups using a simple request-reply protocol that is
 /*	described in \fBtcp_table\fR(5).
-/*	This feature is not included with the stable Postfix release.
+/*	This feature is not included with Postfix 2.2.
 /* .IP "\fBunix\fR (read-only)"
 /*	A limited way to query the UNIX authentication database. The
 /*	following tables are implemented:
@@ -159,21 +117,11 @@
 /*	The result is a group file entry in \fBgroup\fR(5) format.
 /* .RE
 /* .RE
-/* .IP
+/* .sp
 /*	Other table types may exist depending on how Postfix was built.
 /* .IP \fB-n\fR
 /*	Print parameter settings that are not left at their built-in
 /*	default value, because they are explicitly specified in main.cf.
-/* .IP "\fB-t\fR [\fItemplate_file\fR]"
-/*	Display the templates for delivery status notification (DSN)
-/*	messages. To override the built-in templates, specify a
-/*	template file at the end of the command line, or specify a
-/*	template file in main.cf with the \fBbounce_template_file\fR
-/*	parameter.  To force selection of the built-in templates,
-/*	specify an empty template file name (in shell language:
-/*	"").
-/*
-/*	This feature is available with Postfix 2.3 and later.
 /* .IP \fB-v\fR
 /*	Enable verbose logging for debugging purposes. Multiple \fB-v\fR
 /*	options make the software increasingly verbose.
@@ -195,12 +143,9 @@
 /* .IP "\fBconfig_directory (see 'postconf -d' output)\fR"
 /*	The default location of the Postfix main.cf and master.cf
 /*	configuration files.
-/* .IP "\fBbounce_template_file (empty)\fR"
-/*	Pathname of a configuration file with bounce message templates.
 /* FILES
 /*	/etc/postfix/main.cf, Postfix configuration parameters
 /* SEE ALSO
-/*	bounce(5), bounce template file format
 /*	postconf(5), configuration parameters
 /* README FILES
 /* .ad
@@ -232,6 +177,10 @@
 #include <unistd.h>
 #include <ctype.h>
 
+#ifdef STRCASECMP_IN_STRINGS_H
+#include <strings.h>
+#endif
+
 #ifdef USE_PATHS_H
 #include <paths.h>
 #endif
@@ -252,7 +201,6 @@
 #include <vstring_vstream.h>
 #include <myflock.h>
 #include <inet_proto.h>
-#include <argv.h>
 
 /* Global library. */
 
@@ -264,11 +212,6 @@
 #include <mail_params.h>
 #include <mail_addr.h>
 #include <mbox_conf.h>
-#include <mail_run.h>
-
-/* XSASL library. */
-
-#include <xsasl.h>
 
  /*
   * What we're supposed to be doing.
@@ -280,8 +223,6 @@
 #define EDIT_MAIN	(1<<4)		/* edit main.cf */
 #define SHOW_LOCKS	(1<<5)		/* show mailbox lock methods */
 #define SHOW_EVAL	(1<<6)		/* expand right-hand sides */
-#define SHOW_SASL_SERV	(1<<7)		/* show server auth plugin types */
-#define SHOW_SASL_CLNT	(1<<8)		/* show client auth plugin types */
 
  /*
   * Lookup table for in-core parameter info.
@@ -362,7 +303,7 @@ static CONFIG_STR_FN_TABLE str_fn_table_2[] = {
  /*
   * XXX Global so that call-backs can see it.
   */
-static int cmd_mode = SHOW_NAME;
+static int mode = SHOW_NAME;
 
 /* check_myhostname - lookup hostname and validate */
 
@@ -381,12 +322,17 @@ static const char *check_myhostname(void)
     /*
      * If the local machine name is not in FQDN form, try to append the
      * contents of $mydomain.
+     * 
+     * XXX Do not complain when running as "postconf -d".
      */
     name = get_hostname();
-    if ((dot = strchr(name, '.')) == 0) {
-	if ((domain = mail_conf_lookup_eval(VAR_MYDOMAIN)) == 0)
-	    domain = DEF_MYDOMAIN;
-	name = concatenate(name, ".", domain, (char *) 0);
+    if ((mode & SHOW_DEFS) == 0 && (dot = strchr(name, '.')) == 0) {
+	if ((domain = mail_conf_lookup_eval(VAR_MYDOMAIN)) == 0) {
+	    msg_warn("My hostname %s is not a fully qualified name - set %s or %s in %s/main.cf",
+		     name, VAR_MYHOSTNAME, VAR_MYDOMAIN, var_config_dir);
+	} else {
+	    name = concatenate(name, ".", domain, (char *) 0);
+	}
     }
     return (name);
 }
@@ -415,7 +361,7 @@ static const char *check_mydomainname(void)
     if (var_myhostname == 0)
 	get_myhostname();
     if ((dot = strchr(var_myhostname, '.')) == 0 || strchr(dot + 1, '.') == 0)
-	return (DEF_MYDOMAIN);
+	return (var_myhostname);
     return (dot + 1);
 }
 
@@ -427,20 +373,20 @@ static const char *check_mynetworks(void)
     const char *junk;
 
     if (var_inet_interfaces == 0) {
-	if ((cmd_mode & SHOW_DEFS)
-	    || (junk = mail_conf_lookup_eval(VAR_INET_INTERFACES)) == 0)
+	if ((mode & SHOW_DEFS)
+	    || !(junk = mail_conf_lookup_eval(VAR_INET_INTERFACES)))
 	    junk = DEF_INET_INTERFACES;
 	var_inet_interfaces = mystrdup(junk);
     }
     if (var_mynetworks_style == 0) {
-	if ((cmd_mode & SHOW_DEFS)
-	    || (junk = mail_conf_lookup_eval(VAR_MYNETWORKS_STYLE)) == 0)
+	if ((mode & SHOW_DEFS)
+	    || !(junk = mail_conf_lookup_eval(VAR_MYNETWORKS_STYLE)))
 	    junk = DEF_MYNETWORKS_STYLE;
 	var_mynetworks_style = mystrdup(junk);
     }
     if (var_inet_protocols == 0) {
-	if ((cmd_mode & SHOW_DEFS)
-	    || (junk = mail_conf_lookup_eval(VAR_INET_PROTOCOLS)) == 0)
+	if ((mode & SHOW_DEFS)
+	    || !(junk = mail_conf_lookup_eval(VAR_INET_PROTOCOLS)))
 	    junk = DEF_INET_PROTOCOLS;
 	var_inet_protocols = mystrdup(junk);
 	proto_info = inet_proto_init(VAR_INET_PROTOCOLS, var_inet_protocols);
@@ -904,27 +850,13 @@ static void show_maps(void)
 
 static void show_locks(void)
 {
-    ARGV   *locks_argv;
+    ARGV   *maps_argv;
     int     i;
 
-    locks_argv = mbox_lock_names();
-    for (i = 0; i < locks_argv->argc; i++)
-	vstream_printf("%s\n", locks_argv->argv[i]);
-    argv_free(locks_argv);
-}
-
-/* show_sasl - show SASL plug-in types */
-
-static void show_sasl(int what)
-{
-    ARGV   *sasl_argv;
-    int     i;
-
-    sasl_argv = (what & SHOW_SASL_SERV) ? xsasl_server_types() :
-	xsasl_client_types();
-    for (i = 0; i < sasl_argv->argc; i++)
-	vstream_printf("%s\n", sasl_argv->argv[i]);
-    argv_free(sasl_argv);
+    maps_argv = mbox_lock_names();
+    for (i = 0; i < maps_argv->argc; i++)
+	vstream_printf("%s\n", maps_argv->argv[i]);
+    argv_free(maps_argv);
 }
 
 /* show_parameters - show parameter info */
@@ -968,7 +900,6 @@ int     main(int argc, char **argv)
     int     fd;
     struct stat st;
     int     junk;
-    ARGV   *ext_argv = 0;
 
     /*
      * Be consistent with file permissions.
@@ -993,29 +924,17 @@ int     main(int argc, char **argv)
     /*
      * Parse JCL.
      */
-    while ((ch = GETOPT(argc, argv, "aAbc:deEhmlntv")) > 0) {
+    while ((ch = GETOPT(argc, argv, "c:deEhmlnv")) > 0) {
 	switch (ch) {
-	case 'a':
-	    cmd_mode |= SHOW_SASL_SERV;
-	    break;
-	case 'A':
-	    cmd_mode |= SHOW_SASL_CLNT;
-	    break;
-	case 'b':
-	    if (ext_argv)
-		msg_fatal("specify one of -b and -t");
-	    ext_argv = argv_alloc(2);
-	    argv_add(ext_argv, "bounce", "-SVnexpand_templates", (char *) 0);
-	    break;
 	case 'c':
 	    if (setenv(CONF_ENV_PATH, optarg, 1) < 0)
 		msg_fatal("out of memory");
 	    break;
 	case 'd':
-	    cmd_mode |= SHOW_DEFS;
+	    mode |= SHOW_DEFS;
 	    break;
 	case 'e':
-	    cmd_mode |= EDIT_MAIN;
+	    mode |= EDIT_MAIN;
 	    break;
 
 	    /*
@@ -1026,70 +945,41 @@ int     main(int argc, char **argv)
 	     */
 #if 0
 	case 'E':
-	    cmd_mode |= SHOW_EVAL;
+	    mode |= SHOW_EVAL;
 	    break;
 #endif
 	case 'h':
-	    cmd_mode &= ~SHOW_NAME;
+	    mode &= ~SHOW_NAME;
 	    break;
 	case 'l':
-	    cmd_mode |= SHOW_LOCKS;
+	    mode |= SHOW_LOCKS;
 	    break;
 	case 'm':
-	    cmd_mode |= SHOW_MAPS;
+	    mode |= SHOW_MAPS;
 	    break;
 	case 'n':
-	    cmd_mode |= SHOW_NONDEF;
-	    break;
-	case 't':
-	    if (ext_argv)
-		msg_fatal("specify one of -b and -t");
-	    ext_argv = argv_alloc(2);
-	    argv_add(ext_argv, "bounce", "-SVndump_templates", (char *) 0);
+	    mode |= SHOW_NONDEF;
 	    break;
 	case 'v':
 	    msg_verbose++;
 	    break;
 	default:
-	    msg_fatal("usage: %s [-a (server SASL types)] [-A (client SASL types)] [-b (bounce templates)] [-c config_dir] [-d (defaults)] [-e (edit)] [-h (no names)] [-l (lock types)] [-m (map types)] [-n (non-defaults)] [-v] [name...]", argv[0]);
+	    msg_fatal("usage: %s [-c config_dir] [-d (defaults)] [-e (edit)] [-h (no names)] [-l (lock types)] [-m (map types)] [-n (non-defaults)] [-v] [name...]", argv[0]);
 	}
     }
 
     /*
      * Sanity check.
      */
-    junk = (cmd_mode & (SHOW_DEFS | SHOW_NONDEF | SHOW_MAPS | SHOW_LOCKS | EDIT_MAIN | SHOW_SASL_SERV | SHOW_SASL_CLNT));
-    if (junk != 0 && ((junk != SHOW_DEFS && junk != SHOW_NONDEF
-	     && junk != SHOW_MAPS && junk != SHOW_LOCKS && junk != EDIT_MAIN
-		       && junk != SHOW_SASL_SERV && junk != SHOW_SASL_CLNT)
-		      || ext_argv != 0))
-	msg_fatal("specify one of -a, -A, -b, -d, -e, -m, -l and -n");
-
-    /*
-     * Display bounce template information and exit.
-     */
-    if (ext_argv) {
-	if (argv[optind]) {
-	    if (argv[optind + 1])
-		msg_fatal("options -b and -t require at most one template file");
-	    argv_add(ext_argv, "-o",
-		     concatenate(VAR_BOUNCE_TMPL, "=",
-				 argv[optind], (char *) 0),
-		     (char *) 0);
-	}
-	/* Grr... */
-	argv_add(ext_argv, "-o",
-		 concatenate(VAR_QUEUE_DIR, "=", ".", (char *) 0),
-		 (char *) 0);
-	mail_conf_read();
-	mail_run_replace(var_daemon_dir, ext_argv->argv);
-	/* NOTREACHED */
-    }
+    junk = (mode & (SHOW_DEFS | SHOW_NONDEF | SHOW_MAPS | SHOW_LOCKS | EDIT_MAIN));
+    if (junk != 0 && junk != SHOW_DEFS && junk != SHOW_NONDEF
+	&& junk != SHOW_MAPS && junk != SHOW_LOCKS && junk != EDIT_MAIN)
+	msg_fatal("specify one of -d, -e, -m, -l and -n");
 
     /*
      * If showing map types, show them and exit
      */
-    if (cmd_mode & SHOW_MAPS) {
+    if (mode & SHOW_MAPS) {
 	mail_dict_init();
 	show_maps();
     }
@@ -1097,23 +987,14 @@ int     main(int argc, char **argv)
     /*
      * If showing locking methods, show them and exit
      */
-    else if (cmd_mode & SHOW_LOCKS) {
+    else if (mode & SHOW_LOCKS) {
 	show_locks();
     }
 
-    /*
-     * If showing SASL plug-in types, show them and exit
-     */
-    else if (cmd_mode & SHOW_SASL_SERV) {
-	show_sasl(SHOW_SASL_SERV);
-    } else if (cmd_mode & SHOW_SASL_CLNT) {
-	show_sasl(SHOW_SASL_CLNT);
-    }
-
     /*
      * Edit main.cf.
      */
-    else if (cmd_mode & EDIT_MAIN) {
+    else if (mode & EDIT_MAIN) {
 	edit_parameters(argc - optind, argv + optind);
     }
 
@@ -1121,7 +1002,7 @@ int     main(int argc, char **argv)
      * If showing non-default values, read main.cf.
      */
     else {
-	if ((cmd_mode & SHOW_DEFS) == 0) {
+	if ((mode & SHOW_DEFS) == 0) {
 	    read_parameters();
 	    set_parameters();
 	}
@@ -1130,7 +1011,7 @@ int     main(int argc, char **argv)
 	 * Throw together all parameters and show the asked values.
 	 */
 	hash_parameters();
-	show_parameters(cmd_mode, argv + optind);
+	show_parameters(mode, argv + optind);
     }
     vstream_fflush(VSTREAM_OUT);
     exit(0);
