@@ -18,10 +18,9 @@
 /*	QMGR_TRANSPORT *transport;
 /*	void	(*notify)(QMGR_TRANSPORT *transport, VSTREAM *fp);
 /*
-/*	void	qmgr_transport_throttle(transport, dsn, reason)
+/*	void	qmgr_transport_throttle(transport, del_stat)
 /*	QMGR_TRANSPORT *transport;
-/*	const char *dsn;
-/*	const char *reason;
+/*	DEL_STAT *del_stat;
 /*
 /*	void	qmgr_transport_unthrottle(transport)
 /*	QMGR_TRANSPORT *transport;
@@ -135,13 +134,10 @@ void    qmgr_transport_unthrottle(QMGR_TRANSPORT *transport)
 	    msg_info("%s: transport %s", myname, transport->name);
 	transport->flags &= ~QMGR_TRANSPORT_STAT_DEAD;
 	if (transport->dsn == 0)
-	    msg_panic("%s: transport %s: null dsn", myname, transport->name);
-	myfree(transport->dsn);
+	    msg_panic("%s: transport %s: null reason",
+		      myname, transport->name);
+	dsn_free(transport->dsn);
 	transport->dsn = 0;
-	if (transport->reason == 0)
-	    msg_panic("%s: transport %s: null reason", myname, transport->name);
-	myfree(transport->reason);
-	transport->reason = 0;
 	event_cancel_timer(qmgr_transport_unthrottle_wrapper,
 			   (char *) transport);
     }
@@ -149,8 +145,7 @@ void    qmgr_transport_unthrottle(QMGR_TRANSPORT *transport)
 
 /* qmgr_transport_throttle - disable delivery process allocation */
 
-void    qmgr_transport_throttle(QMGR_TRANSPORT *transport, const char *dsn,
-				        const char *reason)
+void    qmgr_transport_throttle(QMGR_TRANSPORT *transport, DSN *dsn)
 {
     char   *myname = "qmgr_transport_throttle";
 
@@ -161,17 +156,13 @@ void    qmgr_transport_throttle(QMGR_TRANSPORT *transport, const char *dsn,
      */
     if ((transport->flags & QMGR_TRANSPORT_STAT_DEAD) == 0) {
 	if (msg_verbose)
-	    msg_info("%s: transport %s: dsn: %s reason: %s",
-		     myname, transport->name, dsn, reason);
+	    msg_info("%s: transport %s: status: %s reason: %s",
+		     myname, transport->name, dsn->status, dsn->reason);
 	transport->flags |= QMGR_TRANSPORT_STAT_DEAD;
 	if (transport->dsn)
-	    msg_panic("%s: transport %s: spurious dsn: %s",
-		      myname, transport->name, transport->dsn);
-	transport->dsn = mystrdup(dsn);
-	if (transport->reason)
 	    msg_panic("%s: transport %s: spurious reason: %s",
-		      myname, transport->name, transport->reason);
-	transport->reason = mystrdup(reason);
+		      myname, transport->name, transport->dsn->reason);
+	transport->dsn = DSN_COPY(dsn);
 	event_request_timer(qmgr_transport_unthrottle_wrapper,
 			    (char *) transport, var_transport_retry_time);
     }
@@ -273,6 +264,7 @@ void    qmgr_transport_alloc(QMGR_TRANSPORT *transport, QMGR_TRANSPORT_ALLOC_NOT
 {
     QMGR_TRANSPORT_ALLOC *alloc;
     VSTREAM *stream;
+    DSN     dsn;
 
     /*
      * Sanity checks.
@@ -301,7 +293,10 @@ void    qmgr_transport_alloc(QMGR_TRANSPORT *transport, QMGR_TRANSPORT_ALLOC_NOT
 
     if ((stream = mail_connect(MAIL_CLASS_PRIVATE, transport->name, BLOCK_MODE)) == 0) {
 	msg_warn("connect to transport %s: %m", transport->name);
-	qmgr_transport_throttle(transport, "4.3.0", "transport is unavailable");
+	qmgr_transport_throttle(transport,
+				DSN_SMTP(&dsn, "4.3.0",
+					 "451 mail transport unavailable",
+					 "mail transport unavailable"));
 	return;
     }
     alloc = (QMGR_TRANSPORT_ALLOC *) mymalloc(sizeof(*alloc));
@@ -372,7 +367,6 @@ QMGR_TRANSPORT *qmgr_transport_create(const char *name)
     transport->candidate_cache_time = (time_t) 0;
     transport->blocker_tag = 1;
     transport->dsn = 0;
-    transport->reason = 0;
     if (qmgr_transport_byname == 0)
 	qmgr_transport_byname = htable_create(10);
     htable_enter(qmgr_transport_byname, name, (char *) transport);
