@@ -9,9 +9,10 @@
 /*	typedef struct {
 /* .in +4
 /*		int code;
-/*		char dsn[...];
+/*		char *dsn;
 /*		char *str;
-/*		VSTRING *buf;
+/*		VSTRING *dsn_buf;
+/*		VSTRING *str_buf;
 /* .in -4
 /*	} SMTP_RESP;
 /*
@@ -37,7 +38,7 @@
 /*	smtp_chat_cmd() formats a command and sends it to an SMTP server.
 /*	Optionally, the command is logged.
 /*
-/*	smtp_chat_resp() read one SMTP server response. It separates the
+/*	smtp_chat_resp() reads one SMTP server response. It separates the
 /*	numerical status code from the text, and concatenates multi-line
 /*	responses to one string, using a newline as separator.
 /*	Optionally, the server response is logged.
@@ -106,7 +107,6 @@
 
 #include "smtp.h"
 
-#define STR(x)	((char *) vstring_str(x))
 #define LEN	VSTRING_LEN
 
 /* smtp_chat_init - initialize SMTP transaction log */
@@ -207,15 +207,17 @@ SMTP_RESP *smtp_chat_resp(SMTP_SESSION *session)
     /*
      * Initialize the response data buffer.
      */
-    if (rdata.buf == 0)
-	rdata.buf = vstring_alloc(100);
+    if (rdata.str_buf == 0) {
+	rdata.dsn_buf = vstring_alloc(10);
+	rdata.str_buf = vstring_alloc(100);
+    }
 
     /*
      * Censor out non-printable characters in server responses. Concatenate
      * multi-line server responses. Separate the status code from the text.
      * Leave further parsing up to the application.
      */
-    VSTRING_RESET(rdata.buf);
+    VSTRING_RESET(rdata.str_buf);
     for (;;) {
 	last_char = smtp_get(session->buffer, session->stream, var_line_limit);
 	printable(STR(session->buffer), '?');
@@ -229,10 +231,10 @@ SMTP_RESP *smtp_chat_resp(SMTP_SESSION *session)
 	 * Defend against a denial of service attack by limiting the amount
 	 * of multi-line text that we are willing to store.
 	 */
-	if (LEN(rdata.buf) < var_line_limit) {
-	    if (VSTRING_LEN(rdata.buf))
-		VSTRING_ADDCH(rdata.buf, '\n');
-	    vstring_strcat(rdata.buf, STR(session->buffer));
+	if (LEN(rdata.str_buf) < var_line_limit) {
+	    if (LEN(rdata.str_buf))
+		VSTRING_ADDCH(rdata.str_buf, '\n');
+	    vstring_strcat(rdata.str_buf, STR(session->buffer));
 	    smtp_chat_append(session, "In:  ", STR(session->buffer));
 	}
 
@@ -259,25 +261,46 @@ SMTP_RESP *smtp_chat_resp(SMTP_SESSION *session)
      * Ignore out-of-protocol enhanced status codes: codes that accompany 3XX
      * replies, or codes whose initial digit is out of sync with the reply
      * code.
+     * 
+     * XXX Potential stability problem. In order to save memory, the queue
+     * manager stores DSNs in a compact manner:
+     * 
+     * - empty strings are represented by null pointers,
+     * 
+     * - the status and reason are required to be non-empty.
+     * 
+     * Other Postfix daemons inherit this behavior, because they use the same
+     * DSN support code. This means that everything that receives DSNs must
+     * cope with null pointers for the optional DSN attributes, and that
+     * everything that provides DSN information must provide a non-empty
+     * status and reason, otherwise the DSN support code wil panic().
+     * 
+     * Thus, when the remote server sends a malformed reply (or 3XX out of
+     * context) we should not panic() in DSN_COPY() just because we don't
+     * have a status. Robustness suggests that we supply a status here, and
+     * that we leave it up to the down-stream code to override the
+     * server-supplied status in case of an error we can't detect here, such
+     * as an out-of-order server reply.
      */
-    DSN_CLASS(rdata.dsn) = 0;
+    VSTRING_TERMINATE(rdata.str_buf);
+    vstring_strcpy(rdata.dsn_buf, "5.5.0");	/* SAFETY! protocol error */
     if (three_digs != 0) {
 	rdata.code = atoi(STR(session->buffer));
 	if (strchr("245", STR(session->buffer)[0]) != 0) {
 	    for (cp = STR(session->buffer) + 4; *cp == ' '; cp++)
 		 /* void */ ;
 	    if ((len = dsn_valid(cp)) > 0 && *cp == *STR(session->buffer)) {
-		DSN_UPDATE(rdata.dsn, cp, len);
+		vstring_strncpy(rdata.dsn_buf, cp, len);
 	    } else {
-		DSN_UPDATE(rdata.dsn, "0.0.0", sizeof("0.0.0") - 1);
-		DSN_CLASS(rdata.dsn) = STR(session->buffer)[0];
+		vstring_strcpy(rdata.dsn_buf, "0.0.0");
+		STR(rdata.dsn_buf)[0] = STR(session->buffer)[0];
 	    }
 	}
     } else {
 	rdata.code = 0;
     }
-    VSTRING_TERMINATE(rdata.buf);
-    rdata.str = STR(rdata.buf);
+    rdata.dsn = STR(rdata.dsn_buf);
+    rdata.str = STR(rdata.str_buf);
     return (&rdata);
 }
 
