@@ -54,9 +54,10 @@
 /* .IP "ATTR_TYPE_DATA (char *, int, char *)"
 /*	This argument is followed by an attribute name, an attribute value
 /*	length, and an attribute value pointer.
-/* .IP "ATTR_TYPE_FUNC (ATTR_PRINT_FN, void *)"
+/* .IP "ATTR_TYPE_FUNC (ATTR_PRINT_SLAVE_FN, void *)"
 /*	This argument is followed by a function pointer and generic data
-/*	pointer.
+/*	pointer. The caller-specified function returns whatever the
+/*	specified attribute printing function returns.
 /* .IP "ATTR_TYPE_HASH (HTABLE *)"
 /* .IP "ATTR_TYPE_NAMEVAL (NVTABLE *)"
 /*	The content of the table is sent as a sequence of string-valued
@@ -114,7 +115,7 @@ int     attr_vprint0(VSTREAM *fp, int flags, va_list ap)
     HTABLE_INFO **ht;
     int     len_val;
     static VSTRING *base64_buf;
-    ATTR_PRINT_FN print_fn;
+    ATTR_PRINT_SLAVE_FN print_fn;
     void   *print_arg;
 
     /*
@@ -168,9 +169,9 @@ int     attr_vprint0(VSTREAM *fp, int flags, va_list ap)
 		msg_info("send attr %s = [data %d bytes]", attr_name, len_val);
 	    break;
 	case ATTR_TYPE_FUNC:
-	    print_fn = va_arg(ap, ATTR_PRINT_FN);
+	    print_fn = va_arg(ap, ATTR_PRINT_SLAVE_FN);
 	    print_arg = va_arg(ap, void *);
-	    print_fn(fp, flags | ATTR_FLAG_MORE, print_arg);
+	    print_fn(attr_print0, fp, flags | ATTR_FLAG_MORE, print_arg);
 	    break;
 	case ATTR_TYPE_HASH:
 	    ht_info_list = htable_list(va_arg(ap, HTABLE *));
