@@ -54,6 +54,10 @@
 /* .IP "ATTR_TYPE_DATA (char *, int, char *)"
 /*	This argument is followed by an attribute name, an attribute value
 /*	length, and a pointer to attribute value.
+/* .IP "ATTR_TYPE_FUNC (ATTR_PRINT_SLAVE_FN, void *)"
+/*	This argument is followed by a function pointer and generic data
+/*	pointer. The caller-specified function returns whatever the
+/*	specified attribute printing function returns.
 /* .IP "ATTR_TYPE_HASH (HTABLE *)"
 /* .IP "ATTR_TYPE_NAMEVAL (NVTABLE *)"
 /*	The content of the table is sent as a sequence of string-valued
@@ -112,6 +116,8 @@ int     attr_vprint_plain(VSTREAM *fp, int flags, va_list ap)
     HTABLE_INFO **ht;
     static VSTRING *base64_buf;
     int     len_val;
+    ATTR_PRINT_SLAVE_FN print_fn;
+    void   *print_arg;
 
     /*
      * Sanity check.
@@ -157,6 +163,11 @@ int     attr_vprint_plain(VSTREAM *fp, int flags, va_list ap)
 	    if (msg_verbose)
 		msg_info("send attr %s = [data %d bytes]", attr_name, len_val);
 	    break;
+	case ATTR_TYPE_FUNC:
+	    print_fn = va_arg(ap, ATTR_PRINT_SLAVE_FN);
+	    print_arg = va_arg(ap, void *);
+	    print_fn(attr_print_plain, fp, flags | ATTR_FLAG_MORE, print_arg);
+	    break;
 	case ATTR_TYPE_HASH:
 	    ht_info_list = htable_list(va_arg(ap, HTABLE *));
 	    for (ht = ht_info_list; *ht; ht++) {
